/*
 * Copyright 2017 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */
package de.frankmuenster.jameica.depot.db.server;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.sql.Time;
import java.util.Date;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.ValueHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * This is the implemtor of the project interface. You never need to instanciate
 * this class directly. Instead of this, use the dbService to find the right
 * implementor of your interface. Example:
 *
 * DBService service = (DBService)
 * Application.getServiceFactory().lookup(ExamplePlugin.class,"JaDepotDB");
 *
 * a) create new project Project project = (Project)
 * service.createObject(Project.class,null);
 *
 * b) load existing project with id "4". Project project = (Project)
 * service.createObject(Project.class,"4");
 *
 * c) list existing projects DBIterator projects =
 * service.createList(Project.class);
 */
/**
 * Implementierung eines Wertpapier-Datensatzes.
 *
 * @author Frank Mnster
 * @version 1.0
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.LongVariable", "PMD.ExcessivePublicCount" })
public class WertpapierImpl extends AbstractDBObject implements Wertpapier {

    /**
     * Aktien_ID ZText fr SQl-Abfragen
     */
    private static final String AKTIEN_ID_SQL = "aktie_id = "; //$NON-NLS-1$

    /**
     * Serial UID
     */
    private static final long serialVersionUID = -4540084930902107527L;

    /**
     * Wertpapier in der ARD Brese
     */
    private transient Boolean inArd;

    /**
     * Implementierung eines Wertpapaier-Datensatzes.
     *
     * @throws RemoteException
     *             {@link java.rmi.RemoteException}
     */
    public WertpapierImpl() throws RemoteException {
        super();
    }

    /**
     * @see Wertpapier#getAdditionalCosts()
     */
    @Override
    public double getAdditionalCosts() throws RemoteException {
        try {
            final DBService service = this.getService();
            final DBIterator<?> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            double additionalCosts = 0.0d;
            while (orderHistory.hasNext()) {
                final OrderHistory order = (OrderHistory) orderHistory.next();
                additionalCosts += order.getAdditionalCosts();
            }
            return additionalCosts;
        } catch (final RemoteException e) {
            throw new RemoteException(Settings.getI18n().tr("WertpapierImpl.calculatingAdditionalCostsFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getAnzahl()
     */
    @Override
    public double getAnzahl() throws RemoteException {
        try {
            final DBService service = this.getService();
            final DBIterator<?> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            double anzahl = 0.0d;
            while (orderHistory.hasNext()) {
                final OrderHistory order = (OrderHistory) orderHistory.next();
                anzahl += order.getOrderAmount();
            }
            return anzahl;
        } catch (final RemoteException e) {
            throw new RemoteException(Settings.getI18n().tr("WertpapierImpl.calculatingNumberOfSharesFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getAttribute(java.lang.String)
     */
    @Override
    public Object getAttribute(final String attribute) throws RemoteException {
        Object attributObject;
        switch (attribute) {
            case "gesamtWert": //$NON-NLS-1$
                attributObject = this.getGesamtWert();
                break;
            case "anzahl": //$NON-NLS-1$
                attributObject = this.getAnzahl();
                break;
            default:
                attributObject = super.getAttribute(attribute);
                break;
        }
        return attributObject;
    }

    /**
     * @see Wertpapier#getChange()
     */
    @Override
    public double getChange() throws RemoteException {
        final Double change = (Double) this.getAttribute("change"); //$NON-NLS-1$
        return change == null || Double.isNaN(change) ? 0.0 : change.doubleValue();
    }

    /**
     * @see Wertpapier#getChangeInPercentage()
     */
    @Override
    public double getChangeInPercentage() throws RemoteException {
        final Double changeInPercentage = (Double) this.getAttribute("changeInPercentage"); //$NON-NLS-1$
        return changeInPercentage == null || Double.isNaN(changeInPercentage) ? 0.0 : changeInPercentage.doubleValue();
    }

    /**
     * @see Wertpapier#getComment()
     */
    @Override
    public String getComment() throws RemoteException {
        return (String) this.getAttribute("comment"); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
     */
    // protected Class getForeignObject(final String foreignKey) throws
    // RemoteException {
    // // We dont have any foreign keys here. Please check TaskImpl.java
    // // if you want to see an example.
    // return super.getForeignObject(foreignKey);
    // }
    /**
     * @see Wertpapier#getCurrency()
     */
    @Override
    public String getCurrency() throws RemoteException {
        return (String) this.getAttribute("currency"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getExchange()
     */
    @Override
    public String getExchange() throws RemoteException {
        return (String) this.getAttribute("exchange"); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#getFfmInstrument()
     */
    @Override
    public String getFfmInstrument() throws RemoteException {
        return (String) this.getAttribute("ffmInstrument"); //$NON-NLS-1$ ;
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#getFfmName()
     */
    @Override
    public String getFfmName() throws RemoteException {
        return (String) this.getAttribute("ffmName"); //$NON-NLS-1$ ;
    }

    /**
     * @see Wertpapier#getGesamtWert()
     */
    @Override
    public double getGesamtWert() throws RemoteException {
        return this.getAnzahl() * this.getLastTradePriceOnly();
    }

    /**
     * @see Wertpapier#getKauf()
     */
    @Override
    @SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
    public double getKauf() throws RemoteException {
        try {
            final DBService service = this.getService();
            final DBIterator<?> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            BigDecimal costs = BigDecimal.ZERO;
            // double costs = 0.0d;
            while (orderHistory.hasNext()) {
                final OrderHistory order = (OrderHistory) orderHistory.next();
                if (order.getOrderAmount() > 0) {
                    final BigDecimal orderPrice = new BigDecimal(order.getOrderPrice());
                    final BigDecimal orderAmount = new BigDecimal(order.getOrderAmount());
                    costs = orderPrice.multiply(orderAmount).add(costs);
                }
            }
            return costs.setScale(2, RoundingMode.HALF_EVEN).doubleValue();
        } catch (final RemoteException e) {
            throw new RemoteException(Settings.getI18n().tr("WertpapierImpl.calculatingExpensesFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getLastTradeDate()
     */
    @Override
    public Date getLastTradeDate() throws RemoteException {
        return (Date) this.getAttribute("lastTradeDate"); //$NON-NLS-1$
    }

    // @Override
    // public double getValue() throws RemoteException {
    // Double d = (Double) getAttribute("value");
    // return d == null || Double.isNaN(d) ? 0.0 : d.doubleValue();
    // }
    /**
     * @see Wertpapier#getLastTradePriceOnly()
     */
    @Override
    public double getLastTradePriceOnly() throws RemoteException {
        // AbstractDBObject will create a java.lang.Double.
        // We only have to cast it.
        final Double lastTradePriceOnly = (Double) this.getAttribute("lastTradePriceOnly"); //$NON-NLS-1$
        return lastTradePriceOnly == null || Double.isNaN(lastTradePriceOnly) ? 0.0 : lastTradePriceOnly.doubleValue();
    }

    /**
     * @see Wertpapier#getLastTradeTime()
     */
    @Override
    public Time getLastTradeTime() throws RemoteException {
        return (Time) this.getAttribute("lastTradeTime"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getName()
     */
    @Override
    public String getName() throws RemoteException {
        // We can cast this directly to String, the method getField() knows the
        // meta data of this sql table ;)
        return (String) this.getAttribute("name"); // "name" ist //$NON-NLS-1$
                                                   // the
                                                   // sql field name
    }

    /**
     * @see Wertpapier#getOrderHistory()
     */
    @Override
    public DBIterator<?> getOrderHistory() throws RemoteException {
        try {
            final DBService service = this.getService();
            final DBIterator<?> orderHistory = service.createList(OrderHistory.class);
            orderHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            orderHistory.setOrder("ORDER BY orderDate desc"); //$NON-NLS-1$
            return orderHistory;
        } catch (final RemoteException e) {
            throw new RemoteException(Settings.getI18n().tr("WertpapierImpl.loadOrderHistoryFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getPreviousClose()
     */
    @Override
    public double getPreviousClose() throws RemoteException {
        // AbstractDBObject will create a java.lang.Double.
        // We only have to cast it.
        final Double previousClose = (Double) this.getAttribute("previousClose"); //$NON-NLS-1$
        return previousClose == null || Double.isNaN(previousClose) ? 0.0 : previousClose.doubleValue();
    }

    /**
     * Sometimes you can display only one of the projects attributes (in combo boxes). Here you can define the name of
     * this field. Please don't confuse this with the "primary KEY".
     *
     * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {
        return "symbol"; //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#getQuandlCode()
     */
    @Override
    public String getQuandlCode() throws RemoteException {
        return (String) this.getAttribute("quandlCode"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getArdId()
     */
    @Override
    public String getArdId() throws RemoteException {
        return (String) this.getAttribute("symbol"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getTotalGewinn()
     */
    @Override
    public double getTotalGewinn() throws RemoteException {
        double gewinn = 0.0d;
        final DBIterator<?> orderHistory = this.getOrderHistory();
        while (orderHistory.hasNext()) {
            final OrderHistory order = (OrderHistory) orderHistory.next();
            // double orderKurs = order.getOrderPrice();
            final double orderGewinn = (this.getLastTradePriceOnly() - order.getOrderPrice()) * order.getOrderAmount();
            if (Logger.isLogging(Level.DEBUG)) {
                if (order.getOrderAmount() > 0) {
                    Logger.debug(
                            Settings.getI18n().tr("WertpapierImpl.orderProfitPurchase", Double.toString(orderGewinn))); //$NON-NLS-1$
                } else if (order.getOrderAmount() < 0) {
                    Logger.debug(Settings.getI18n().tr("WertpapierImpl.orderProfitSale", Double.toString(orderGewinn))); //$NON-NLS-1$
                }
            }
            gewinn += orderGewinn;
        }
        return gewinn;
    }

    /**
     * @see Wertpapier#getType()
     */
    @Override
    public String getType() throws RemoteException {
        return (String) this.getAttribute("type"); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#getValueHistory()
     */
    @Override
    public DBIterator<?> getValueHistory() throws RemoteException {
        try {
            final DBService service = this.getService();
            final DBIterator<?> valueHistory = service.createList(ValueHistory.class);
            valueHistory.addFilter(WertpapierImpl.AKTIEN_ID_SQL + this.getID());
            return valueHistory;
        } catch (final RemoteException e) {
            throw new RemoteException(Settings.getI18n().tr("WertpapierImpl.loadHistoryValuesFailed"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Wertpapier#getVerkauf()
     */
    @Override
    public double getVerkauf() throws RemoteException {
        double realisiert = 0.0d;
        final DBIterator<?> orderHistory = this.getOrderHistory();
        while (orderHistory.hasNext()) {
            final OrderHistory order = (OrderHistory) orderHistory.next();
            if (order.getOrderAmount() < 0) {
                // Nur Verkufe bercksichtigen
                realisiert += order.getOrderPrice() * order.getOrderAmount() * -1;
            }
        }
        return realisiert;
    }

    /**
     * @see Wertpapier#getYearHigh()
     */
    @Override
    public double getYearHigh() throws RemoteException {
        final Double yearHigh = (Double) this.getAttribute("yearHigh"); //$NON-NLS-1$
        return yearHigh == null || Double.isNaN(yearHigh) ? 0.0 : yearHigh.doubleValue();
    }

    /**
     * @see Wertpapier#getYearLow()
     */
    @Override
    public double getYearLow() throws RemoteException {
        final Double yearLow = (Double) this.getAttribute("yearLow"); //$NON-NLS-1$
        return yearLow == null || Double.isNaN(yearLow) ? 0.0 : yearLow.doubleValue();
    }

    /**
     * @see Wertpapier#isInArd()
     */
    @Override
    public boolean isInArd() throws RemoteException {
        if (this.inArd == null) {
            // Aktienabfrage starten
            try {
                ArdStockInfoBean stockInfo = new ArdStockInfoBean();
                stockInfo.setId(this.getArdId());
                stockInfo = BoerseArdApi.getInstance().getActualValue(stockInfo);
                // wenn wir hier sind, gibt es das Wertpapier
                if (stockInfo != null) {
                    this.inArd = true;
                }
            } catch (final RemoteException e) {
                this.inArd = false;
            } catch (final FinanceException e) {
                this.inArd = false;
            }
        }
        return this.inArd;
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setInArd()
     */
    @Override
    public void setInArd() {
        this.inArd = null; // NOPMD
    }

    /**
     * @see Wertpapier#parseWertpapier(StockInfoBean)
     */
    @Override
    public void parseWertpapier(final StockInfoBean wpi) throws RemoteException {
        this.setChange(wpi.getChangeRate().doubleValue());
        // setSymbol(wpi.getSymbol());
        this.setCurrency(wpi.getCurrency());
        this.setExchange(wpi.getExchange());
        this.setLastTradeDate(wpi.getDate());
        this.setLastTradeTime(new Time(wpi.getDate().getTime()));
        this.setLastTradePriceOnly(wpi.getActValue().doubleValue());
        // setName(wpi.getName());
        this.setPreviousClose(Double.NaN);
        this.setChangeInPercentage(wpi.getChangeRate().doubleValue());
    }

    /**
     * @see Wertpapier#setChange(double)
     */
    @Override
    public void setChange(final double change) throws RemoteException {
        this.setAttribute("change", change); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setChangeInPercentage(double)
     */
    @Override
    public void setChangeInPercentage(final double changeInPercentage) throws RemoteException {
        this.setAttribute("changeInPercentage", changeInPercentage); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setComment(String)
     */
    @Override
    public void setComment(final String comment) throws RemoteException {
        this.setAttribute("comment", comment); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setCurrency(String)
     */
    @Override
    public void setCurrency(final String currency) throws RemoteException {
        this.setAttribute("currency", currency); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setExchange(String)
     */
    @Override
    public void setExchange(final String exchange) throws RemoteException {
        this.setAttribute("exchange", exchange); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setFfmInstrument(java.lang.String)
     */
    @Override
    public void setFfmInstrument(final String ffmInstrument) throws RemoteException {
        this.setAttribute("ffmInstrument", ffmInstrument); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setFfmName(java.lang.String)
     */
    @Override
    public void setFfmName(final String ffmName) throws RemoteException {
        this.setAttribute("ffmName", ffmName); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setLastTradeDate(Date)
     */
    @Override
    public void setLastTradeDate(final Date lastTradeDate) throws RemoteException {
        this.setAttribute("lastTradeDate", lastTradeDate); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setLastTradePriceOnly(double)
     */
    @Override
    public void setLastTradePriceOnly(final double lastTradePriceOnly) throws RemoteException {
        this.setAttribute("lastTradePriceOnly", new Double(lastTradePriceOnly)); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setLastTradeTime(Time)
     */
    @Override
    public void setLastTradeTime(final Time lastTradeTime) throws RemoteException {
        this.setAttribute("lastTradeTime", lastTradeTime); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setName(String)
     */
    @Override
    public void setName(final String name) throws RemoteException {
        this.setAttribute("name", name == null ? StringUtils.EMPTY : name); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setPreviousClose(double)
     */
    @Override
    public void setPreviousClose(final double previousClose) throws RemoteException {
        this.setAttribute("previousClose", new Double(previousClose)); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.depot.db.rmi.Wertpapier#setQuandlCode(java.lang.String)
     */
    @Override
    public void setQuandlCode(final String quandlCode) throws RemoteException {
        this.setAttribute("quandlCode", quandlCode); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setArdId(String)
     */
    @Override
    public void setArdId(final String ardId) throws RemoteException {
        this.setAttribute("symbol", ardId); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setType(String)
     */
    @Override
    public void setType(final String type) throws RemoteException {
        this.setAttribute("type", type); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setYearHigh(double)
     */
    @Override
    public void setYearHigh(final double yearHigh) throws RemoteException {
        this.setAttribute("yearHigh", yearHigh); //$NON-NLS-1$
    }

    /**
     * @see Wertpapier#setYearLow(double)
     */
    @Override
    public void setYearLow(final double yearLow) throws RemoteException {
        this.setAttribute("yearLow", yearLow); //$NON-NLS-1$
    }

    /**
     * This method will be called, before delete() is executed. Here you can make some dependency checks. If you dont
     * want to delete the project (in case of failed dependencies) you have to throw an ApplicationException. The
     * message of this one will be shown in users UI. So please translate the text into the users language.
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
     */
    @Override
    protected void deleteCheck() throws ApplicationException {
        // Maybe used later
    }

    /**
     * We have to return the name of the sql table here.
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
     */
    @Override
    protected String getTableName() {
        return "wertpapier"; //$NON-NLS-1$
    }

    /**
     * This method is invoked before executing insert(). So lets check the entered data.
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    protected void insertCheck() throws ApplicationException {
        try {
            if (this.getArdId() == null || this.getArdId().length() == 0) {
                throw new ApplicationException(Settings.getI18n().tr("WertpapierImpl.symbolNullOrEmpty")); //$NON-NLS-1$
            }
        } catch (final RemoteException e) {
            Logger.error(Settings.getI18n().tr("WertpapierImpl.insertCheckFailed"), e); //$NON-NLS-1$
            throw (ApplicationException) new ApplicationException(
                    Settings.getI18n().tr("WertpapierImpl.insertCheckFailedMsg")) //$NON-NLS-1$
                            .initCause(e);
        }
    }

    /**
     * This method is invoked before every update().
     *
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {
        // we simply call the insertCheck here ;)
        this.insertCheck();
    }
}
