//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.finanzen;

import java.rmi.RemoteException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmWertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlWertpapierHistory;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.finanzen.WertpapierHistory;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class WertpapierHistoryBean {

    /**
     *
     */
    public static final String DAX_ID = "159096"; //$NON-NLS-1$

    /**
     * Datenquelle
     */
    private transient String dataSource;

    /**
     * DatenAbfrage erfolgreich
     */
    private transient boolean dataRequestOK;

    /**
     * Anfangsdatum der Werte
     */
    private transient LocalDate fromDate;

    /**
     * Die Wertpapier ID
     */
    private transient String wpId;

    /**
     * Enddatum der Werte
     */
    private transient LocalDate toDate;

    /**
     * Liefert eine Liste mit den historischen Werten des Wertpapiers
     */
    private transient List<HistoryValueBean> wphv = new ArrayList<>();

    /**
     * Konstruktor mit Angabe der ARD ID, des Quandl-Codes und der FFM Brsennames
     *
     * @param wpId
     * @param quandlCode
     * @param ffmInfoBean
     * @throws FinanceException
     */
    public WertpapierHistoryBean(final String wpId, final String quandlCode, final FfmStockInfoBean ffmInfoBean)
            throws FinanceException {
        // In privater Methode, damit sie vom 2.ten Konstruktor mit verwendet
        // werden kann.
        this.init(wpId, quandlCode, ffmInfoBean);
    }

    /**
     * Konstruktor mit Angabe eines Wertpapier-Okjektes
     *
     * @param wertpapier
     * @throws FinanceException
     */
    public WertpapierHistoryBean(final Wertpapier wertpapier) throws FinanceException {
        String wpId = null;
        try {
            wpId = wertpapier.getArdId();
        } catch (final RemoteException e) { // NOPMD
            // Hier machen wir nix
        }
        String quandlCode = null;
        try {
            quandlCode = wertpapier.getQuandlCode();
        } catch (final RemoteException e) { // NOPMD
            // Und hier auch nix
        }
        FfmStockInfoBean ffmInfoBean = null;
        try {
            ffmInfoBean = new FfmStockInfoBean(wertpapier.getFfmInstrument(), wertpapier.getFfmName(), null, null);
        } catch (final RemoteException e) { // NOPMD
            // Und hier auch nix
        }
        // Und nun die eigentliche Arbeit
        this.init(wpId, quandlCode, ffmInfoBean);
    }

    /**
     * Liefert die Datenquelle
     *
     * @return Datenquelle
     */
    public String getDataSource() {
        return this.dataSource;
    }

    /**
     * Liefert das Anfangsdatum
     *
     * @return fromDate
     */
    public LocalDate getFromDate() {
        return this.fromDate;
    }

    /**
     * Liefert eine Liste mit den historischen Werten des Wertpapiers.
     *
     * @return wphv Liste mit den HistoryValueBean
     */
    public List<HistoryValueBean> getHistoryValues() {
        return this.wphv;
    }

    /**
     * Liefert die Wertpapier ID
     *
     * @return wpId
     */
    public String getwpId() {
        return this.wpId;
    }

    /**
     * Liefert die historischen Werte f&uuml;r eine angegeben Zeitraum.
     *
     * @param fromDate
     *            das Anfangsdatum
     * @param toDate
     *            das Enddatum
     * @return Die Liste mit HistoryValueBean
     */
    public List<HistoryValueBean> getTimeRangeValues(final LocalDate fromDate, final LocalDate toDate) {
        final List<HistoryValueBean> rangeValues = new ArrayList<>();
        for (final HistoryValueBean value : this.wphv) {
            final LocalDate valueDate = value.getLocalDate();
            if (valueDate.compareTo(fromDate) >= 0 && valueDate.compareTo(toDate) <= 0) {
                rangeValues.add(value);
            }
        }
        return rangeValues;
    }

    /**
     * Liefert das Enddatum
     *
     * @return toDate
     */
    public LocalDate getToDate() {
        return this.toDate;
    }

    /**
     * Liefert die historischen Werte f&uuml;r ein angegebenes Datum. Falls kein Daten f&uuml; das angegeben Datum zur
     * Verf&uuml;gung stehen, wird der n&auml;chste davorliegende Handelstag genommen. Liegt kein Tage vor DATUM, werden
     * die Daten des ersten zur Verf&uuml;gung stehenden Tages verwendet. Liegen gar keine Daten vor wir NULL
     * zur&uuml;ckgegeben.
     *
     * @param date
     *            das Datum
     * @return Der HistoryValueBean
     */
    public HistoryValueBean getValuesAtDate(final LocalDate date) {
        HistoryValueBean returnValue = null;
        if (!this.wphv.isEmpty()) {
            returnValue = this.wphv.get(0);
            for (final HistoryValueBean value : this.wphv) {
                final LocalDate dateValue = value.getLocalDate();
                if (dateValue.isEqual(date)) {
                    // Gesuchtes Datum gefunden
                    returnValue = value;
                    break;
                } else if (date.isBefore(dateValue)) {
                    // Wir sind am gesuchten Datum vorbei
                    break;
                }
                returnValue = value;
            }
        }
        return returnValue;
    }

    /**
     * War der Datenabruf OK.
     *
     * @return the dataRequestOK
     */
    public Boolean isDataRequestOK() {
        return this.dataRequestOK;
    }

    /**
     * @param wpId
     * @param quandlCode
     * @param ffmInfoBean
     * @throws FinanceException
     */
    private void init(final String wpId, final String quandlCode, final FfmStockInfoBean ffmInfoBean)
            throws FinanceException {
        this.wpId = wpId;
        this.dataRequestOK = false;
        this.dataSource = StringUtils.EMPTY;
        // Mal bei Quandl versuchen
        if (StringUtils.isNotEmpty(quandlCode)) {
            this.loadHistory(quandlCode);
        }
        if (this.wphv.isEmpty() && ffmInfoBean != null) {
            // probieren wir es mal mit der Brse FFM
            this.loadFfm(ffmInfoBean);
        }
        if (this.wphv.isEmpty() && StringUtils.isNotEmpty(wpId)) {
            // probieren wir es mal mit der ARD Brse
            this.loadArdBoerse();
        }
        if (this.wphv.isEmpty()) {
            this.dataSource = StringUtils.EMPTY;
            throw new FinanceException("Keine historischen Daten gefunden", FinanceExceptionType.DATA_NOT_FOUND); //$NON-NLS-1$
        }
        this.dataRequestOK = true;
        // Dann sortieren.
        Collections.sort(this.wphv);
        // und das Anfangs- und Enddatum setzten
        this.fromDate = this.wphv.get(0).getLocalDate();
        this.toDate = this.wphv.get(this.wphv.size() - 1).getLocalDate();
    }

    /**
     *
     */
    private void loadArdBoerse() {
        try {
            final ArdStockInfoBean stockInfo = new ArdStockInfoBean();
            stockInfo.setId(this.wpId);
            BoerseArdApi.getInstance().loadHistoryValues(stockInfo);
            this.wphv = stockInfo.getHistoryValues().getValues();
            this.dataSource = stockInfo.getHistoryValues().getSourceName();
        } catch (final FinanceException ex) {
            // Hier nur loggen
            Logger.error(ex.getLocalizedMessage(), ex);
        }
    }

    /**
     * @param ffmInfoBean
     */
    private void loadFfm(final FfmStockInfoBean ffmInfoBean) {
        try {
            final WertpapierHistory fwh = new FfmWertpapierHistory(ffmInfoBean);
            this.wphv = fwh.getValues();
            this.dataSource = fwh.getSourceName();
        } catch (final FinanceException ex) {
            // Hier nur loggen
            Logger.error(ex.getLocalizedMessage(), ex);
        }
    }

    /**
     * @param quandlCode
     */
    private void loadHistory(final String quandlCode) {
        try {
            final WertpapierHistory qwh = new QuandlWertpapierHistory(quandlCode, Settings.getQuandlApiKey());
            this.wphv = qwh.getValues();
            this.dataSource = qwh.getSourceName();
        } catch (final FinanceException ex) {
            // Hier nur loggen
            Logger.error(ex.getLocalizedMessage());
        }
    }
}
