//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.finanzen.fixer_io;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.time.DateUtils;

import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.CurrencyConverter;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.ExchangeRate;
import de.frankmuenster.jameica.depot.finanzen.fixer_io.json.ForeignExchanges;
import de.frankmuenster.jameica.depot.finanzen.fixer_io.json.Rates;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class FixerIOCurrencyConverter implements CurrencyConverter {

    /**
     * Datumsformat bei Fixer.IO
     */
    private static final String PATTERN = "yyyy-MM-dd"; //$NON-NLS-1$

    /**
     * Die interne Instanz
     */
    private static FixerIOCurrencyConverter instance;

    // String test = "http://api.fixer.io/2000-01-03?symbols=EUR,BGN";
    /**
     * Liefert die Instanz des FixerIO-W&auml;hrungsumrechner.
     *
     * @return die Instanz des FixerIO-W&auml;hrungsumrechner.
     */
    public static FixerIOCurrencyConverter getInstance() {
        synchronized (FixerIOCurrencyConverter.class) {
            if (FixerIOCurrencyConverter.instance == null) {
                FixerIOCurrencyConverter.instance = new FixerIOCurrencyConverter();
            }
        }
        return FixerIOCurrencyConverter.instance;
    }

    /**
     * Privater Konstruktor um externe Instanziierung zu verhindern.
     */
    private FixerIOCurrencyConverter() {
    }

    /**
     * @see de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.
     *      CurrencyConverter#exchangeRate(java.lang.String, java.lang.String)
     */
    @Override
    public ExchangeRate exchangeRate(final String currencyFrom, final String currencyTo) throws FinanceException {
        final ExchangeRate exRate = this.exchangeRate(currencyFrom, currencyTo, null);
        exRate.setExcatDate(true);
        return exRate;
    }

    /**
     * @see de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.
     *      CurrencyConverter#exchangeRate(java.lang.String, java.lang.String, java.util.Date)
     */
    @Override
    public ExchangeRate exchangeRate(final String currencyFrom, final String currencyTo, final Date date)
            throws FinanceException {
        final Calendar cal = Calendar.getInstance(Locale.getDefault());
        cal.set(2000, 0, 1, 0, 0, 0);
        final Date dateFirst = cal.getTime();
        if (date != null && date.before(dateFirst)) {
            throw new FinanceException(I18N.tr("FixerIOCurrencyConverter.dateTooOld"), //$NON-NLS-1$
                    FinanceExceptionType.DATE_TOO_OLD);
        }
        final SimpleDateFormat sdf = new SimpleDateFormat(FixerIOCurrencyConverter.PATTERN, Locale.GERMAN);
        String dateString = "latest"; //$NON-NLS-1$
        Date requestDate = date;
        if (requestDate == null) {
            requestDate = new Date();
        } else {
            dateString = sdf.format(requestDate);
        }
        String request = ""; //$NON-NLS-1$
        URL url;
        Float test = Float.NaN;
        try {
            request = FixerIOWebAPI.MAIN_URL + dateString + "?base=" + currencyFrom; //$NON-NLS-1$
            url = new URL(request);
            final String responseString = FixerIOWebAPI.getInstance().getResonseGETRequest(url, true);
            final ForeignExchanges exchanges = FixerIOWebAPI.MAPPER.readValue(responseString, ForeignExchanges.class);
            Logger.debug(exchanges.getDate());
            final Rates rates = exchanges.getRates();
            final Map<String, Object> map = rates.getChangeRates();
            final Double dblRate = (Double) map.get(currencyTo);
            if (dblRate == null) {
                Logger.error(I18N.tr("FixerIOCurrencyConverter.currencyNotSupported")); //$NON-NLS-1$
                throw new FinanceException(I18N.tr("FixerIOCurrencyConverter.currencyNotSupported"), //$NON-NLS-1$
                        FinanceExceptionType.CURRENCY_NOT_SUPPORTED);
            }
            final Date rateDay = sdf.parse(exchanges.getDate());
            test = new BigDecimal(dblRate.toString()).floatValue();
            final ExchangeRate exRate = new ExchangeRate(rateDay, new BigDecimal(test.toString()));
            exRate.setExcatDate(DateUtils.isSameDay(requestDate, rateDay));
            return exRate;
        } catch (final MalformedURLException e) {
            throw new FinanceException(I18N.tr("WebAPI.malformedUrl", request), //$NON-NLS-1$
                    FinanceExceptionType.OTHER, e);
        } catch (final IOException e) {
            Logger.error(I18N.tr("FixerIOCurrencyConverter.jsonDeserialisationFailed"), e); //$NON-NLS-1$
            throw new FinanceException(I18N.tr("FixerIOCurrencyConverter.jsonDeserialisationFailed"), //$NON-NLS-1$
                    FinanceExceptionType.DATA_NOT_FOUND, e);
        } catch (final ParseException e) {
            Logger.error(I18N.tr("FixerIOCurrencyConverter.changeDateNotReadable"), e); //$NON-NLS-1$
            throw new FinanceException(I18N.tr("FixerIOCurrencyConverter.changeDateNotReadable"), //$NON-NLS-1$
                    FinanceExceptionType.DATA_NOT_FOUND, e);
        }
    }
}
