//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.time.LocalDate;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.jimmoores.quandl.HeaderDefinition;
import com.jimmoores.quandl.Row;
import com.jimmoores.quandl.TabularResult;

import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class HistoryReader {

    /**
     * Date Namen
     */
    private final static List<String> DATE = new ArrayList<>(Arrays.asList("Date")); //$NON-NLS-1$

    /**
     * Open Namen
     */
    private final static List<String> OPEN = new ArrayList<>(Arrays.asList("Open", "Previous Day Price", "USD (AM)")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

    /**
     * Close Namen
     */
    private final static List<String> CLOSE = new ArrayList<>(Arrays.asList("Close", "Last", "Value", "USD (PM)")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

    /**
     * High Namen
     */
    private final static List<String> HIGH = new ArrayList<>(Arrays.asList("High")); //$NON-NLS-1$

    /**
     * Low Namen
     */
    private final static List<String> LOW = new ArrayList<>(Arrays.asList("Low")); //$NON-NLS-1$

    /**
     * Volume Namen
     */
    private final static List<String> VOLUME = new ArrayList<>(Arrays.asList("Trade Volume", "Volume")); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Aufzhlung der Spaltennamen
     */
    private enum ColNames {
        DATE,
        OPEN,
        CLOSE,
        HIGH,
        LOW,
        VOLUME
    }

    /**
     * Standardkonstruktor
     */
    private HistoryReader() {
        // Absichtlich leer
    }

    /**
     * List die historischen Werte aus einem TabularResult
     *
     * @param idString
     * @param result
     * @return Liste mit den Werten
     */
    public static List<HistoryValueBean> resultReader(final String idString, final TabularResult result) {

        final List<HistoryValueBean> values = new ArrayList<>();
        if (result != null) {
            final Map<ColNames, Integer> indexMap = HistoryReader.findIndexes(result.getHeaderDefinition());
            HistoryValueBean value;
            for (int i = 0; i < result.size(); i++) {
                final Row row = result.get(i);
                value = new HistoryValueBean(idString); // NOPMD
                try {
                    final LocalDate date = LocalDate.parse(row.getString(indexMap.get(ColNames.DATE)));
                    value.setLocalDate(date);
                } catch (final DateTimeParseException e) {
                    Logger.error("Fehler beim Lesen des Datums.", e); //$NON-NLS-1$
                }
                if (indexMap.get(ColNames.OPEN) >= 0) {
                    final Double open = row.getDouble(indexMap.get(ColNames.OPEN));
                    value.setOpen(HistoryReader.doubelOrNan(open));
                }
                if (indexMap.get(ColNames.CLOSE) >= 0) {
                    final Double close = row.getDouble(indexMap.get(ColNames.CLOSE));
                    value.setClose(HistoryReader.doubelOrNan(close));
                }
                if (indexMap.get(ColNames.HIGH) >= 0) {
                    final Double high = row.getDouble(indexMap.get(ColNames.HIGH));
                    value.setHigh(HistoryReader.doubelOrNan(high));
                }
                if (indexMap.get(ColNames.LOW) >= 0) {
                    final Double low = row.getDouble(indexMap.get(ColNames.LOW));
                    value.setLow(HistoryReader.doubelOrNan(low));
                }
                if (indexMap.get(ColNames.VOLUME) >= 0) {
                    final Double low = row.getDouble(indexMap.get(ColNames.VOLUME));
                    value.setVolume(HistoryReader.doubelOrNan(low));
                }
                values.add(value);
            }
        }
        return values;
    }

    /**
     * Ermittelt die Spaltenindexes
     *
     * @param headerInfo
     * @return Map mit den Indexes
     */
    private static Map<ColNames, Integer> findIndexes(final HeaderDefinition headerInfo) {

        final Map<ColNames, Integer> map = new ConcurrentHashMap<>();
        map.put(ColNames.DATE, HistoryReader.getIndexOf(headerInfo, HistoryReader.DATE));
        map.put(ColNames.OPEN, HistoryReader.getIndexOf(headerInfo, HistoryReader.OPEN));
        map.put(ColNames.CLOSE, HistoryReader.getIndexOf(headerInfo, HistoryReader.CLOSE));
        map.put(ColNames.HIGH, HistoryReader.getIndexOf(headerInfo, HistoryReader.HIGH));
        map.put(ColNames.LOW, HistoryReader.getIndexOf(headerInfo, HistoryReader.LOW));
        map.put(ColNames.VOLUME, HistoryReader.getIndexOf(headerInfo, HistoryReader.VOLUME));
        return map;
    }

    /**
     * Ermittelt den Index der Spalte
     *
     * @param headerInfo
     *            die HeaderInfos
     * @param titels
     *            der mglichen Spaltennamen
     * @return den Index null basiert oder -1, falls die Spalte nicht existiert.
     */
    private static int getIndexOf(final HeaderDefinition headerInfo, final List<String> titels) {

        int index = -1;
        for (final String columnTitel : titels) {
            try {
                index = headerInfo.columnIndex(columnTitel);
            } catch (final IllegalArgumentException e) { // NOPMD
                // Nix zu tun hier
            }
        }
        return index;
    }

    /**
     * Wandelt ein <b>Double</b> Objekt in einen <b>double</b> Wert um. Gibt
     * <b>Double.NaN</b> wenn <i>doubleValue</i> <b>NULL</b> ist.
     *
     * @param doubleValue
     *            das Double-Objekt
     * @return der double Wert oder Double.NaN, wenn doubleValue NULL ist.
     */
    private static double doubelOrNan(final Double doubleValue) {

        return doubleValue == null ? Double.NaN : doubleValue;
    }
}
