/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.controller;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.action.WertpapierDetailAction;
import de.frankmuenster.jameica.depot.gui.action.listener.WertpapierAktualisiertListener;
import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.gui.parts.WertpapierListePart;
import de.frankmuenster.jameica.depot.gui.parts.column.ExchangeInfo;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.logging.Logger;

/**
 * Kontroller f&uumr; die Hauptseite
 *
 * @author Frank Mnster
 */
public class DepotController extends AbstractControl implements WertpapierAktualisiertListener {

    /**
     * Konstante die einen Double Null Wert reprsentiert. Wird zur Initialisierung von BigDecimal verwendet.
     */
    private static final String DOUBLE_ZERO = "0.0"; //$NON-NLS-1$

    /**
     * Wechselkurs Info
     */
    private transient TextAreaInput wechselKursInfo;

    /**
     * Hibiscus Saldo
     */
    private transient LabelInputExtended hibiscusTotal;

    /**
     * JaDepot Saldo
     */
    private transient LabelInputExtended jaDepotTotal;

    /**
     * Hibiscus + JaDepot Saldo
     */
    private transient LabelInputExtended hibisJaDepotTotal;

    /**
     * Liste der Wertpapiere
     */
    private transient WertpapierListePart wpListePart;

    /**
     * Konstruktor mit View
     *
     * @param view
     */
    public DepotController(final AbstractView view) {
        super(view);
    }

    /**
     * Liefert die Tabelle mit den Wertpapieren.
     *
     * @return the wpListePart
     */
    public WertpapierListePart getAktienListe() {
        if (this.wpListePart == null) {
            try {
                this.wpListePart = new WertpapierListePart(new WertpapierDetailAction());
            } catch (final RemoteException e) {
                Logger.error(e.getMessage());
            }
        }
        return this.wpListePart;
    }

    /**
     * Liefert die Wechselkurs Infoanzeige.
     *
     * @return TextAreaInput wechselKursInfo
     */
    public TextAreaInput getWechselKursInfos() {
        if (this.wechselKursInfo == null) {
            this.wechselKursInfo = new TextAreaInput(StringUtils.EMPTY);
            this.wechselKursInfo.setComment(StringUtils.EMPTY);
            this.wechselKursInfo.setEnabled(false);
            this.wechselKursInfo.setHeight(50);
        }
        this.wechselKursInfo.setValue(this.getWechselKursInfo());
        return this.wechselKursInfo;
    }

    /**
     * Liefert die Saldoanzeige f&uuml; die Hibiscus-Konten
     *
     * @return LabelInputExtended hibiscusTotal
     */
    public LabelInputExtended getHibiscusTotal() {
        if (this.hibiscusTotal == null) {
            this.hibiscusTotal = new LabelInputExtended(StringUtils.EMPTY, true, SWT.RIGHT);
            this.hibiscusTotal.setComment(Settings.getDepotWaehrung());
            this.hibiscusTotal.setEnabled(false);
        }
        final BigDecimal hbTotal = this.calculateHibiscusTotal();
        this.hibiscusTotal.setValue(" " //$NON-NLS-1$
                + FormatterUtils.DECIMAL_FORMAT.get().format(hbTotal));
        return this.hibiscusTotal;
    }

    /**
     * Liefert die Summe der JaDepot-Wertpapiere
     *
     * @return LabelInputExtended jaDepotTotal
     */
    public LabelInputExtended getJaDepotTotal() {
        if (this.jaDepotTotal == null) {
            this.jaDepotTotal = new LabelInputExtended(StringUtils.EMPTY, true, SWT.RIGHT);
            this.jaDepotTotal.setComment(Settings.getDepotWaehrung());
            this.jaDepotTotal.setEnabled(false);
        }
        this.jaDepotTotal.setValue(" " //$NON-NLS-1$
                + FormatterUtils.DECIMAL_FORMAT.get().format(this.getAktienListe().getGesamtSumme()));
        return this.jaDepotTotal;
    }

    /**
     * Liefert die Summe der Salden.
     *
     * @return LabelInputExtended hibisJaDepotTotal
     */
    public LabelInputExtended getHibiscusJaDepotTotal() {
        if (this.hibisJaDepotTotal == null) {
            this.hibisJaDepotTotal = new LabelInputExtended(StringUtils.EMPTY, true, SWT.RIGHT);
            this.hibisJaDepotTotal.setComment(Settings.getDepotWaehrung());
            this.hibisJaDepotTotal.setEnabled(false);
        }
        final BigDecimal total = this.calculateTotal();
        this.hibisJaDepotTotal.setValue(" " + FormatterUtils.DECIMAL_FORMAT.get().format(total)); //$NON-NLS-1$
        return this.hibisJaDepotTotal;
    }

    /**
     * @see WertpapierAktualisiertListener #notifyWertpapiereAktualisiert()
     */
    @Override
    public void notifyWertpapiereAktualisiert() {
        // die Tabelle neu laden
        final WertpapierListePart wpListe = this.getAktienListe();
        try {
            wpListe.removeAll();
            JaDepotDBService service;
            service = (JaDepotDBService) Settings.getDBService();
            final DBIterator<?> list = service.createList(Wertpapier.class);
            list.setOrder("ORDER BY name asc"); //$NON-NLS-1$
            while (list.hasNext()) {
                final Wertpapier wertpapier = (Wertpapier) list.next();
                wpListe.addItem(wertpapier);
            }
            this.refreshTotals();
        } catch (final RemoteException e) {
            Logger.error(e.getMessage());
        }
    }

    /**
     * Aktualisiert die Totals
     */
    private void refreshTotals() {
        // Wechselkurse
        this.getWechselKursInfos();
        // JaDepot Total
        final BigDecimal jaTotal = this.getAktienListe().getGesamtSumme();
        this.jaDepotTotal.setValue(" " //$NON-NLS-1$
                + FormatterUtils.DECIMAL_FORMAT.get().format(jaTotal));
        // Hibiscus Total
        final BigDecimal hbTotal = this.calculateHibiscusTotal();
        this.hibiscusTotal.setValue(" " //$NON-NLS-1$
                + FormatterUtils.DECIMAL_FORMAT.get().format(hbTotal));
        // Und das gesamte Total
        final BigDecimal total = jaTotal.add(hbTotal);
        this.hibisJaDepotTotal.setValue(" " + FormatterUtils.DECIMAL_FORMAT.get().format(total)); //$NON-NLS-1$
    }

    private String getWechselKursInfo() {
        final StringBuffer waehrungInfo = new StringBuffer();
        if (this.getAktienListe().isShowWaehrungsSpalte()) {
            final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
                    Locale.GERMAN);
            final DecimalFormat dateFormat = new DecimalFormat("###,###,##0.0000"); //$NON-NLS-1$
            final List<ExchangeInfo> exchangeInfos = this.getAktienListe().getExchangeInfos();
            for (final ExchangeInfo exchangeInfo : exchangeInfos) {
                waehrungInfo.append(exchangeInfo.getWaehrungFrom()).append(' ')
                        .append(Settings.getI18n().tr("DepotController.to")) //$NON-NLS-1$
                        .append(' ').append(exchangeInfo.getWaehrungTo()).append(":   ") //$NON-NLS-1$
                        .append(dateFormat.format(exchangeInfo.getKurs()));
                if (exchangeInfo.getDatum() != null) { // NOPMD
                    waehrungInfo.append(", ") //$NON-NLS-1$
                            .append(Settings.getI18n().tr("DepotController.date")) //$NON-NLS-1$
                            .append(": ") //$NON-NLS-1$
                            .append(sdf.format(exchangeInfo.getDatum()));
                }
                waehrungInfo.append('\n');
            }
        }
        return waehrungInfo.toString();
    }

    private BigDecimal calculateHibiscusTotal() {
        BigDecimal hibiscusTotal = new BigDecimal(DepotController.DOUBLE_ZERO);
        if (Settings.isHibiscusInstalled()) {
            try {
                final DBIterator<?> hibiscusKontos = de.willuhn.jameica.hbci.Settings.getDBService()
                        .createList(de.willuhn.jameica.hbci.rmi.Konto.class);
                // i1.setOrder("ORDER BY LOWER(kategorie), blz, kontonummer,
                // bezeichnung");
                while (hibiscusKontos.hasNext()) {
                    final de.willuhn.jameica.hbci.rmi.Konto konto = (de.willuhn.jameica.hbci.rmi.Konto) hibiscusKontos
                            .next();
                    Logger.debug(Settings.getI18n().tr("DepotController.logAccountBalance", //$NON-NLS-1$
                            konto.getName(), Double.toString(konto.getSaldo())));
                    hibiscusTotal = hibiscusTotal.add(new BigDecimal(konto.getSaldo())); // NOPMD
                }
            } catch (final RemoteException e) {
                Logger.error(Settings.getI18n().tr("DepotController.getBalanceError" + e.getMessage())); //$NON-NLS-1$
                hibiscusTotal = new BigDecimal(DepotController.DOUBLE_ZERO);
            }
        } else {
            hibiscusTotal = new BigDecimal(DepotController.DOUBLE_ZERO);
        }
        return hibiscusTotal;
    }

    /**
     * Berechnet die Summe aus Hibiscus-Konten und den Wertpapieren. Die Local wird dabei beachtet...
     *
     * @return die Summe
     */
    private BigDecimal calculateTotal() {
        // Calculate total
        BigDecimal total = new BigDecimal(DepotController.DOUBLE_ZERO);
        if (Settings.isHibiscusInstalled()) {
            String totalString = (String) this.hibiscusTotal.getValue();
            // Check for comma as decimal separator and replace
            final DecimalFormatSymbols dfs = FormatterUtils.DECIMAL_FORMAT.get().getDecimalFormatSymbols();
            final String grpSeparator = String.valueOf(dfs.getGroupingSeparator());
            final String deciSeparator = String.valueOf(dfs.getDecimalSeparator());
            totalString = totalString.replace(grpSeparator, StringUtils.EMPTY);
            if (!".".equals(deciSeparator)) { // NOPMD //$NON-NLS-1$
                totalString = totalString.replace(deciSeparator, "."); //$NON-NLS-1$
            }
            final BigDecimal hbTotal = new BigDecimal(totalString.trim());
            total = total.add(hbTotal);
        }
        total = total.add(this.getAktienListe().getGesamtSumme());
        return total;
    }
}
