/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.utils;

import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * Utiliy Klasse fr Datumsberechnungen
 *
 * @author Frank Mnster
 */
public final class DateUtils {

    /**
     * Millisekunden eines Tages
     */
    private static final double DAY_MILLIS = 24.0 * 60.0 * 60.0 * 1000.0;

    /**
     * Standardkonstruktor
     */
    private DateUtils() {
        // absichlich leer
    }

    /**
     * Addiert die Anzahl an Tagen und liefert ein neues Objekt basierend auf Now()
     *
     * @param days
     *            Anzahl die addiert wird (kann negative sein)
     * @return das new Date Objekt mit der Anzahl addiert
     */
    public static Date addDays(final int days) {
        final Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, days);
        return cal.getTime();
    }

    /**
     * Subtrahiert die Anzahl an Tagen und liefert ein neues Objekt basierend auf Now()
     *
     * @param days
     *            Anzahl die addiert wird (kann negative sein)
     * @return das new Date Objekt mit der Anzahl subtrahiert
     */
    public static LocalDate subtractDays(final int days) {
        return LocalDate.now().minusDays(days);
    }

    /**
     * Berechnet die Differenz zwischen Jetzt und dem Date-Objekt
     *
     * @param date
     *            das Date-Objekt, nicht Null
     * @return die Differenz Tagen
     */
    public static long dateDiff(final LocalDate date) {
        return ChronoUnit.DAYS.between(date, LocalDate.now());
    }

    /**
     * Berechnet die Differenz zwischen Jetzt und dem Date-Objekt
     *
     * @param date
     *            das Date-Objekt, nicht Null
     * @return die Differenz Tagen
     */
    public static long dateDiff(final LocalDateTime date) {
        return ChronoUnit.DAYS.between(date, LocalDateTime.now());
    }

    /**
     * Berechnet die Differenz zwischen den beiden Date-Objekt
     *
     * @param fromDate
     *            das Anfangsdatum Date-Objekt, nicht Null
     * @param toDate
     *            das Enddatum Date-Objekt, nicht Null
     * @return die Differenz in Tagen
     */
    public static long dateDiff(final Date fromDate, final Date toDate) {
        return (long) ((toDate.getTime() - fromDate.getTime()) / DateUtils.DAY_MILLIS);
    }

    /**
     * Berechnet die Differenz zwischen den beiden Date-Objekt
     *
     * @param fromDate
     *            das Anfangsdatum Date-Objekt, nicht Null
     * @param toDate
     *            das Enddatum Date-Objekt, nicht Null
     * @return die Differenz in Tagen
     */
    public static long dateDiff(final LocalDate fromDate, final LocalDate toDate) {
        return ChronoUnit.DAYS.between(fromDate, toDate);
    }

    /**
     * Liefert das Jahr des Datums als Integerwert.
     *
     * @param date
     *            das Datum
     * @return das Jahr als Integerwert
     */
    public static int getYear(final Date date) {
        final Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        return calendar.get(Calendar.YEAR);
    }

    /**
     * Liefert den Monat des Datums als Integerwert. Januar gleich 1, Dezember gleich 12.
     *
     * @param date
     *            das Datum
     * @return den Monat als Integerwert
     */
    public static int getMonth(final Date date) {
        final Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        return calendar.get(Calendar.MONTH + 1);
    }

    /**
     * Liefert den Tag des Datums als Integerwert.
     *
     * @param date
     *            das Datum
     * @return den Tag als Integerwert
     */
    public static int getDay(final Date date) {
        final Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        return calendar.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * @param localDate
     * @return Date object from LocalDate Object
     */
    public static Date asDate(final LocalDate localDate) {
        return Date.from(localDate.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * @param localDateTime
     * @return Date object from LocalDateTime object
     */
    public static Date asDate(final LocalDateTime localDateTime) {
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * @param date
     * @return LocalDate object from Date object
     */
    public static LocalDate asLocalDate(final Date date) {
        return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
    }

    /**
     * @param date
     * @return LocalDateTime object from Date object
     */
    public static LocalDateTime asLocalDateTime(final Date date) {
        return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    /**
     * Checks, if a date is in rage of two other dates.
     *
     * @param date
     *            date to be checked, not null
     * @param fromDate
     *            first of the range, not null
     * @param toDate
     *            last of the range, not null
     * @return true, if fromDate <= date <= toDate, false else
     */
    public static boolean isInRange(final LocalDate date, final LocalDate fromDate, final LocalDate toDate) {
        return fromDate.compareTo(date) <= 0 && toDate.compareTo(date) >= 0;
    }
}
