/*
 * Copyright 2014 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse;

import java.time.LocalDate;
import java.util.List;

import org.eclipse.swt.graphics.Color;
import org.jfree.chart.plot.XYPlot;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;

/**
 * Abstrakte Klasse fr Analysemethoden.
 *
 * @param <P>
 *            Der Type des KursAnalyseParameters
 * @param <R>
 *            Der Ergebnistyp. Darf nur KursDaten[] oder XYPLot sein.
 * @author Frank Mnster
 */
public abstract class AbstractKursAnalyse<P, R> implements IKursAnalyse<P, R> {

    /**
     * Die Parameter fr die Analysemethode
     */
    protected transient KursAnalyseParameter<P> parameter;

    /**
     * Der Ergebnis XYPLot
     */
    protected XYPlot anaG;

    /**
     * Die hostorischen Werte des Wertpapiers.
     */
    protected transient WertpapierHistoryBean wertPapierHistory;

    /**
     * Die Schlusskurse
     */
    protected transient KursDaten closeDaten = new KursDaten();

    // private String legendTitel = null;
    /**
     * Anzahl der Linien
     */
    private int lineCount;

    /**
     * Die Farben der Linien
     */
    private Color[] color;

    /**
     * Der Style der Linien
     */
    private KursAnalyseLineStyle[] lineStyle;

    /**
     * Soll der Bereich unterhalb ausgemalt werden.
     */
    private boolean enableArea;

    /**
     * Line las Steps zeichen
     */
    private boolean enableStep;

    /**
     * Der Typ der Analyse
     */
    private KursAnalyseTyp type;

    /**
     * Standardkonstruktor
     */
    public AbstractKursAnalyse() {
        // Absichlich leer
    }

    /**
     * Konstruktor mit Angaben von WertpapterHistory und den Analyseparametern.
     *
     * @param wpHistoryBean
     * @param parameter
     */
    public AbstractKursAnalyse(final WertpapierHistoryBean wpHistoryBean, final KursAnalyseParameter<P> parameter) {
        this.wertPapierHistory = wpHistoryBean;
        final List<HistoryValueBean> wphv = wpHistoryBean.getHistoryValues();
        // Loop thru all Values
        for (final HistoryValueBean historyValue : wphv) {
            final DateValuePair value = new DateValuePair( // NOPMD
                    historyValue.getLocalDate(), historyValue.getClose());
            this.closeDaten.add(value);
        }
        this.parameter = parameter;
    }

    /**
     * @return legendTitel
     */
    @Override
    public abstract String getLegendTitel();

    /**
     * @return color
     */
    @Override
    public Color[] getColor() {
        return this.color.clone();
    }

    /**
     * Setzt den Linienstile
     */
    @Override
    public void setLineStyle(final KursAnalyseLineStyle... lineStyle) {
        this.lineStyle = lineStyle.clone();
    }

    /**
     * @return color
     */
    @Override
    public KursAnalyseLineStyle[] getLineStyle() {
        return this.lineStyle.clone();
    }

    /**
     * Setzt die Anzahl der Linien im Kurstyp OVERLAY
     *
     * @return Anzahl Linien
     */
    @Override
    public int getLineCount() {
        return this.type == KursAnalyseTyp.SEPERATE_GRAPH ? 0 : this.lineCount;
    }

    /**
     * Setzt die Anzahl an Kurslinien im Kurstyp OVERLAY
     *
     * @param count
     *            die Anzahl an Linien
     */
    @Override
    public void setLineCount(final int count) {
        this.lineCount = count;
    }

    /**
     * @param color
     *            das zu setzende Objekt color
     */
    @Override
    public void setColor(final Color... color) {
        this.color = color.clone();
    }

    /**
     * @return enableArea
     */
    public boolean isEnableArea() {
        return this.enableArea;
    }

    /**
     * @param enableArea
     *            das zu setzende Objekt enableArea
     */
    public void setEnableArea(final boolean enableArea) {
        this.enableArea = enableArea;
    }

    /**
     * @return enableStep
     */
    public boolean isEnableStep() {
        return this.enableStep;
    }

    /**
     * @param enableStep
     *            das zu setzende Objekt enableStep
     */
    public void setEnableStep(final boolean enableStep) {
        this.enableStep = enableStep;
    }

    /**
     * @see IKursAnalyse#setWertpapierHistory(WertpapierHistoryBean)
     */
    @Override
    public void setWertpapierHistory(final WertpapierHistoryBean wpHistoryBean) {
        this.wertPapierHistory = wpHistoryBean;
        this.closeDaten.clear();
        final List<HistoryValueBean> wphv = wpHistoryBean.getHistoryValues();
        // Loop thru all Values
        for (final HistoryValueBean historyValue : wphv) {
            final DateValuePair value = new DateValuePair( // NOPMD
                    historyValue.getLocalDate(), historyValue.getClose());
            this.closeDaten.add(value);
        }
    }

    /**
     * @param parameter
     */
    @Override
    public void setKursAnalyseParameter(final KursAnalyseParameter<P> parameter) {
        this.parameter = parameter;
    }

    /**
     * @see IKursAnalyse#getType()
     */
    @Override
    public KursAnalyseTyp getType() {
        return this.type;
    }

    /**
     * @see IKursAnalyse#setType(KursAnalyseTyp)
     */
    @Override
    public void setType(final KursAnalyseTyp type) {
        this.type = type;
    }

    /**
     * @see IKursAnalyse#getResult(LocalDate, LocalDate)
     */
    @Override
    public abstract R getResult(LocalDate fromDate, LocalDate toDate);
}
